@@ -1,11 +1,93 @@
+#' Convert between degrees and radians
+#' 
+#' Facilitates conversion between degrees and radians.
+#' @rdname deg2rad
+#' @return a numeric vector
+#' @param x a numeric vector
+#' @export
+#' @examples
+#' deg2rad(180)
+#' 
+
+deg2rad <- function(x) {
+	x/180 * base::pi
+}
+
+#' @rdname deg2rad
+#' @export
+#' @examples
+#' rad2deg(2*pi)
+#' @seealso \code{\link{latlon2xyz}}, \code{\link{googleMap}}, and \code{\link{rgeo}}.
+
+
+rad2deg <- function(x) {
+	x / base::pi * 180
+}
+
+#' Convert back and forth between latitude/longitude and XYZ-space
+#' 
+#' @rdname latlon2xyz
+#' @return a matrix each row of which describes the latitudes and longitudes
+#' @param x,y,z numeric vectors
+#' @export
+#' @examples
+#' xyz2latlon(1, 1, 1)     # point may be on sphere of any radius
+#' xyz2latlon(0, 0, 0)     # this produces a NaN for latitude
+
+xyz2latlon <- function(x,y,z) {
+	# rescale to unit sphere
+	R <- sqrt( x^2 + y^2 + z^2)
+	x <- x/R; y <- y/R; z <- z/R;
+
+	r <- sqrt( x^2 + y^2)
+	lat <- rad2deg( asin(z) )
+
+	long <- (r>0) * rad2deg( acos(x/r) ) 
+	long <- ( 1 - 2 * (y < 0) ) * long
+	long [ is.na(long) ] <- 0
+	return( cbind(lat=lat, lon=long) )
+}
+
+#' @rdname latlon2xyz
+#' @param latitude,longitude vectors of latitude and longitude values
+#' @return a matrix each row of which contains the x, y, and z coordinates of a point on a unit sphere
+#' 
+#' @export
+#' @examples
+#' latlon2xyz(45, 45)
+#' @seealso \code{\link{deg2rad}}, \code{\link{googleMap}}, and \code{\link{rgeo}}.
+
+latlon2xyz <- function(latitude,longitude) {
+	z <- sin(deg2rad(latitude))
+	r <- sqrt(1 - z^2)
+	x <- rad2deg(cos( longitude ))
+	y <- rad2deg(sin( longitude ))
+	return(cbind( x=x, y=y, z=z ))
+}
+
 #' Sample longitude and latitude on a sphere
 #' 
 #' Randomly samples longitude and latitude on earth so that equal areas are
 #' (approximately) equally likely to be sampled.  
 #' (Approximation assumes earth as a perfect sphere.)
 #' 
-#' Some additional utility functions are documented here as well.
+#' @rdname rgeo
+#' @param n number of random locations
+#' 
+#' @param latlim,lonlim  
+#'   range of latitudes and longitudes to sample within, only implemented for \code{rgeo}.
+#' 
+#' @param verbose 
+#'   return verbose output that includes Euclidean coordinates on unit sphere as well as 
+#' longitude and latitude.
 #'
+#' @return a data frame with variables \code{long} and \code{lat}.  If \code{verbose} is
+#' TRUE, then x, y, and z coordinates are also included in the data frame.
+#' @export
+#' @examples
+#' rgeo(4)
+#' # sample from a region that contains the continental US
+#' rgeo( 4, latlim=c(25,50), lonlim=c(-65,-125) )
 #' @details
 #' \code{rgeo} and \code{rgeo2} differ in the algorithms used to generate random positions.  
 #' Each assumes a spherical globe.  \code{rgeo} uses that fact that each of the x, y and z
@@ -18,95 +100,13 @@
 #' discards any point outside the sphere contained in the cube and projects the non-discarded points
 #' to the sphere.  This method must oversample to allow for the discarded points.
 #' 
-#' @author Randall Pruim (\email{rpruim@@calvin.edu})
-#' 
-#' @seealso \code{\link{googleMap}}
+#' @seealso \code{\link{deg2rad}}, \code{\link{googleMap}} and \code{\link{latlon2xyz}}.
 #' 
 #' @keywords random 
 #' @keywords geometry 
+#' @keywords map 
 #' 
 
-#' @param x a numeric vector
-#'
-#' @return a numeric vector 
-#'
-#' @rdname rgeo
-#' @export
-#' @examples
-#' deg2rad(180)
-#' 
-
-deg2rad <- function(x) {
-	x/180 * base::pi
-}
-
-#' @return a numeric vector
-#' 
-#' @rdname rgeo
-#' @examples
-#' rad2deg(2*pi)
-
-rad2deg <- function(x) {
-	x / base::pi * 180
-}
-
-#' @param y,z numeric vectors
-#' @return a matrix each row of which contains a latitude and a longitude value
-#'
-#' @rdname rgeo
-#' @examples
-#' xyz2latlon(1,1,1)     # point may be on sphere of any radius
-#' xyz2latlon(0,0,0)     # this produces a NaN for latitude
-xyz2latlon <- function(x,y,z) {
-
-	# rescale to unit sphere
-	R <- sqrt( x^2 + y^2 + z^2)
-	x <- x/R; y <- y/R; z <- z/R;
-
-	r <- sqrt( x^2 + y^2)
-	lat <- rad2deg( asin(z) )
-
-
-	long <- (r>0) * rad2deg( acos(x/r) ) 
-	long <- ( 1 - 2 * (y < 0) ) * long
-	long [ is.na(long) ] <- 0
-	return( cbind(lat=lat, lon=long) )
-}
-
-#' @param latitude,longitude vectors of latitude and longitude values
-#' @return a matrix each row of which contains the x, y, and z coordinates of a point on a unit sphere
-#' 
-#' @rdname rgeo
-#' @examples
-#' latlon2xyz(45,45)
-
-latlon2xyz <- function(latitude,longitude) {
-	z <- sin(deg2rad(latitude))
-	r <- sqrt(1 - z^2)
-	x <- rad2deg(cos( longitude ))
-	y <- rad2deg(sin( longitude ))
-	return(cbind( x=x, y=y, z=z ))
-}
-
-#' @param n 
-#'   number of random locations
-#' 
-#' @param latlim,lonlim  
-#'   range of latitudes and longitudes to sample within, only implemented for \code{rgeo}.
-#' 
-#' @param verbose 
-#'   return verbose output that includes Euclidean coordinates on unit sphere as well as 
-#' longitude and lattitude.
-#'
-#' @return a data frame with variables \code{long} and \code{lat}.  If \code{verbose} is
-#' TRUE, then x, y, and z coordinates are also included in the data frame.
-#' 
-#' @rdname rgeo
-#' @examples
-#' rgeo(4)
-#' # sample from a region that contains the continental US
-#' rgeo( 4, latlim=c(25,50), lonlim=c(-65,-125) )
-
 rgeo <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {
 
 	zlim <- sin(sort(deg2rad(latlim)))
@@ -129,9 +129,7 @@ rgeo <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {
 }
 
 #' @rdname rgeo
-#' @return a data frame with variables \code{long} and \code{lat}.  If \code{verbose} is
-#' TRUE, then x, y, and z coordinates are also included in the data frame.  
-#'
+#' @export
 #' @examples
 #' rgeo2(4)
 rgeo2 <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {
@@ -162,8 +160,13 @@ rgeo2 <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {
 	}
 	return(data.frame(lat=latlon[,1], lon=latlon[,2]))
 }
+#' Display a point on earth on a Google Map
+#' 
+#' Creates a URL for Google Maps for a particular latitude and
+#' longitude position.
 
-#' @rdname rgeo
+#' @rdname googleMap
+#' @param latitude,longitude vectors of latitude and longitude values
 #' @param position a data frame containing latitude and longitude positions
 #' @param zoom zoom level for initial map (1-20)
 #' @param maptype one of \code{'roadmap'}, \code{'satellite'}, \code{'terrain'}, and \code{'hybrid'}
@@ -172,6 +175,8 @@ rgeo2 <- function( n=1, latlim=c(-90,90), lonlim=c(-180,180), verbose=FALSE ) {
 #' @param browse a logical indicating whether the URL should be browsed (else only returned as a string)
 #' @param \dots additional arguments passed to \code{browseURL}
 #' @return a string containing a URL.  Optionally, as a side-effect, the URL is visited in a browser
+#' @export
+#' @seealso \code{\link{deg2rad}}, \code{\link{latlon2xyz}} and \code{\link{rgeo}}.
 
 googleMap <- function(latitude, longitude, position=NULL,
 	zoom=12, 
@@ -224,7 +229,7 @@ googleMap <- function(latitude, longitude, position=NULL,
 		url <- file.path(paths, filename)
 		url <- paste("file://",url,sep="")
 	}
-	url <- paste('http://mosaic-web.org/',filename,sep="")
+	url <- paste('http://mosaic-web.org/go/',filename,sep="")
 	if (is.null(position)) {
 		position <- data.frame(lat=latitude,lon=longitude)
 	}
